; ************************************************************************
; ************* CDROM Steuer- und Kontrollprogramm                      **
; ************* (C) 1999-03-31 by Klaus Peichl                          **
; *************                   Dornbergstr. 10                       **
; *************                   91058 Erlangen-Tennenlohe             **
; *************                   (pei@freenet.de)                      **
; ************************************************************************

		.MODEL tiny
		.Code
		.386
		ORG 100h


; ************************************************************************
; ************* Makros                                                  **
; ************************************************************************


; ------------------------------------------------------------------------
  GetDrive      MACRO
		LOCAL DriveOk, CheckOk
; ------------------------------------------------------------------------
; Ermittelt den Laufwerksbuchstaben des CDROM-Laufwerks
; Rckgabe:     cx    = Laufwerk (0=A, 1=B, 2=C ...)
;               Error = 0 oder 1

		push eax
		push ebx
		mov ax,1500h
		int 2Fh
		mov Error,0
		cmp bl,Unit      ; Anzahl der CDROM-Laufwerke
		ja  DriveOk
		mov Error,100
;                PRINT CDROM-Error=100[device_not_present]
;                NewLine
  DriveOk:
                add cl,Unit

; Der folgende Installationstest unterbricht bei manchen Treibern
; die Wiedergabe, deshalb ist er deaktiviert:
;		mov  ax,150Bh                ; check device
;		int  2Fh
;		cmp  bx,0ADADh
;		je   CheckOk
;		mov Error,101
;               PRINT CDROM-Error=101[device_error]
;               NewLine

  CheckOk:      pop ebx
		pop eax

		;mov al,cl
		;add al,'A'
		;mov Letter,al
		;mov dx,offset Letter
		;mov ah,9
		;int 21h

		ENDM


; ------------------------------------------------------------------------
  ClearReqHdr   MACRO
; ------------------------------------------------------------------------
		; Request Header zunchst mit Nullen fllen:
		push ax
		push cx
		push di

		mov ax,ds
		mov es,ax

		mov di,Offset ReqHeader+1
		mov cx,12+33
		cld
		mov al,0
	    rep stosb
		pop di
		pop cx
		pop ax

		ENDM

; ------------------------------------------------------------------------
  Print         MACRO S
		LOCAL Weiter
		LOCAL StringMem
; ------------------------------------------------------------------------
		push dx
		push ds
		push cs
		pop ds
		mov dx,Offset StringMem
		call PrintString
		jmp Weiter
		StringMem DB '&S','$'

  Weiter:       pop ds
		pop dx
		
		ENDM

; ------------------------------------------------------------------------
  NewLine       MACRO
; ------------------------------------------------------------------------
; Gibt die CR/LF-Sequenz aus
		push dx
		mov dx,Offset CR
		call PrintString
		pop dx
		ENDM

; ------------------------------------------------------------------------
  GetStabe      MACRO
; ------------------------------------------------------------------------
                lodsb
		and al,255-32           ; Klein -> Grobuchstabe
		ENDM


; ************************************************************************
; ************* Hauptprogramm                                           **
; ************************************************************************


Start:          push ds
		pop  es

		mov si,81h
  SuchStabe:    GetStabe
		jz  SuchStabe

                cmp al,15               ; Slash 0x2f '/' converted to capital
                jne n_Slash
                GetStabe
                cmp al,'U'              ; Unit (0=erstes CD-Laufwerk)
                jnz n_Slash
                call ReadParamNumber
                mov Unit,al
                jmp SuchStabe
n_Slash:

		cmp al,'I'
		jne n_I

		; Parameter "INFO":
		call PrintInfoZeile
		jmp Ende

  n_I:          cmp al,'T'
		jne n_T

                ; Parameter "TRACK":
    SuchZahl2:  mov al,[si]
                cmp al,'+'
                je Plus2
                cmp al,'-'
                je Minus2
                cmp al,13
                ja  weiter2
                call GetQInfo
                call SetTrack
                jmp Ende
    weiter2:    sub al,'0'             ; ASCII -> Byte-Wert
                cmp al,9
                jbe Zahl2
                inc si
                jmp SuchZahl2
    Zahl2:      call ReadParamNumber
                call SetTrack
                jmp Ende
    Plus2:      call GetQInfo
                inc al
                call SetTrack
                jmp Ende
    Minus2:     call GetQInfo
                dec al
                call SetTrack
                jmp Ende

  n_T:          cmp al,'R'
		jne n_R

		; Parameter "RESET":
		call ResetDrive
		jmp Ende
  
  n_R:          cmp al,'P'
                jne n_P

                ; Parameter "PLAY":
    SuchZahl:   mov al,[si]
                cmp al,'+'
                je Plus
                cmp al,'-'
                je Minus
                cmp al,13
                ja  weiter
                call GetQInfo
                call PlayTrack
                jmp Ende
    weiter:     sub al,'0'             ; ASCII -> Byte-Wert
                cmp al,9
                jbe Zahl
                inc si
                jmp SuchZahl
    Zahl:       call ReadParamNumber
                call PlayTrack
                jmp Ende
    Plus:       call FastForward
                jmp Ende
    Minus:      call FastReverse
                jmp Ende

  n_P:          cmp al,'S'
                jne n_S

                GetStabe
                cmp al,'T'
                jne n_ST
                GetStabe
                cmp al,'A'
                jne n_STA

                ; Parameter "STATUS":
                call GetAudioStatus
                and AudioStatus,1       ; Pause-Bit extrahieren
                mov ebx,EndSektor       ; ebx = Endsektor
                call GetQInfo
                call QInfo2Time
                call Time2Sector        ; eax = aktueller Sektor
                cmp eax,ebx
                jae PlayVorbei
                mov ax,AudioStatus
                jmp StatusEnd
  PlayVorbei:   mov al,2
  StatusEnd:    jmp Ende2

    n_STA:      ; Parameter "STOP":
                call StopAudio
    n_ST:       jmp Ende

  n_S:          cmp al,'C'
		jne n_C

                GetStabe
                cmp al,'O'
                jne n_CO

		; Parameter "CONT":
		call ResumeAudio
		jmp Ende

    n_CO:       ; Parameter "CLOSE":
		call Close
		jmp Ende

  n_C:          cmp al,'V'
                jne n_V
                call PrintInfos
                jmp Ende

  n_V:          cmp al,'D'
                jne n_D

                ; Parameter "DETECT":
                call GetStatus
                test ax,2048
                jz DiskDa
                mov al,1                ; EL=1 => no Disc
                jmp Ende
    DiskDa:     mov al,1
                call GetTrackStart
                mov al,Byte Ptr [ControlCode+6]  ; Info-Byte
                and al,64               ; Bit fr Audio/Data
                shr al,6
                inc al
                inc al
                jmp Ende

  n_D:          cmp al,'E'
                jne n_E

                ; Parameter "EJECT":
                call Eject
                jmp Ende

  n_E:          cmp al,'L'
                jne n_L

                ; Parameter "LONGNAME":
    ShortName:  mov di,Offset PathShortName
    ScanName:   lodsb
                cmp al,13               ; Dateiname zu Ende?
                jbe NameTerm
                cmp al,' '
                jbe ScanName
                cmp al,':'              ; Es war ein Laufwerksbuchstabe
                je ShortName            ; vorangestellt => berschreiben
                stosb
                jmp ScanName
    NameTerm:   mov al,0
                stosb
                call GetLongName
                mov dx,Offset LongName
                mov ah,9
                int 21h
                jmp Ende

  n_L:          cmp al,'G'
                jne n_G
                GetStabe
                cmp al,'E'
                jne n_G
                GetStabe
                cmp al,'T'
                jne n_G
                GetStabe
                cmp al,'F'
                je GetFirst
                cmp al,'L'
                je GetLast

                ; Parameter "GETTRACK":
                call GetQInfo
                jmp Ende2

                ; Parameter "GETLAST":
    GetLast:    call GetTracks
                mov al,LastTrack
                jmp Ende2

                ; Parameter "GETFIRST":
    GetFirst:   call GetTracks
                mov al,FirstTrack
                jmp Ende2

  n_G:
		; ...

  SyntaxError:  mov dx,Offset SyntaxStr
                call PrintString

Ende:           cmp Error,0
                jz Ende2
                mov al,Error
Ende2:          mov ah,4ch
		int 21h



; ************************************************************************
; ************* Unterprogramme                                          **
; ************************************************************************


; ------------------------------------------------------------------------
  PrintInfos:
; ------------------------------------------------------------------------
; Gibt einige Kenndaten der CD aus

                PRINT Sectorsize=
                call GetSectorSize
                call PrintHexWord
	        NewLine

	        call GetTracks
	        PRINT  FirstTrack=
	        mov al,firstTrack
	        call PrintHexByte
                NewLine

	        PRINT  LastTrack=
	        mov al,lastTrack
	        call PrintHexByte
                NewLine

                call GetStatus
                PRINT Status=
                call PrintHexWord
	        NewLine

                ret


; ------------------------------------------------------------------------
  PlayTrack:
; ------------------------------------------------------------------------
; Spielt genau einen Track
; Eingabe:      al = Track

		push ax                 ; Track-Nr. retten
		call GetTracks
		cmp al,LastTrack
		jae IsLast
		; Anfang des nchsten Tracks in eax ermitteln:
		inc ax
		call GetTrackStart
		call Time2Sector

		jmp CalcLen
  IsLast:       mov eax,LeadOut
		call Time2Sector
  CalcLen:      mov ebx,eax
		pop ax                  ; eigentliche Track-Nr. zurckholen
		call GetTrackStart
		call Time2Sector
		sub ebx,eax             ; ebx=Lnge
		call PlayAudio

                ret


; ------------------------------------------------------------------------
  PlayTrackAt:
; ------------------------------------------------------------------------
; Spielt den Rest des aktuellen Tracks ab Sektor eax
; Eingabe:      Track (vorher GetQInfo aufrufen)
;               eax = Startsektor

		push eax                ; Startsektor retten
                mov al,Track
		call GetTracks
		cmp al,LastTrack
		jae IsLast2
		; Anfang des nchsten Tracks in eax ermitteln:
		inc ax
		call GetTrackStart
		call Time2Sector

		jmp CalcLen2
  IsLast2:      mov eax,LeadOut
		call Time2Sector

  CalcLen2:     mov ebx,eax             ; ebx = Endsektor
		pop eax                 ; eax = Startsektor zurckholen
                cmp eax,ebx
                jae TrackOver
                call StopAudio
		sub ebx,eax             ; ebx=Lnge
		call PlayAudio
  TrackOver:
                ret



; ------------------------------------------------------------------------
  ReadParamNumber:
; ------------------------------------------------------------------------
; Liest einen in Ziffern geschriebenen numerischen Wert von der Kommando-
; zeile und wandelt den String in eine Zahl um
; Eingabe:      si = Startindex in der Kommandozeile
; Rckgabe:     ax = Zahl

                push bx
                push cx

                mov cx,10
                xor ax,ax               ; mov ax,0
                xor bx,bx               ; mov bx,0

  Hole_Zeichen: mov bl,[si]
                inc si
                sub bl,'0'             ; ASCII -> Byte-Wert
                cmp bl,'9'
                ja keine_Ziffer
                mul cx                  ; ax mu 10-fach gewichtet werden
                add ax,bx
                jmp Hole_Zeichen
  keine_Ziffer:
                pop cx
                pop bx
                ret


; ------------------------------------------------------------------------
  GetLongName:
; ------------------------------------------------------------------------
; Bestimmt zu einem kurzen (8.3) Dateinamen mit Pfad auf CDROM den
; dazugehrigen langen Dateinamen.
; Eingabe:      Pfad und kurzer Dateiname bei Offset PathShortName (nullt.)
; Rckgabe:     langer Dateiname bei Offset LongName (nullt.)

		GetDrive

                ; Fr den Fall, da das Auslesen des langen Dateinamens
                ; fehlschlgt, wird zunchst der 8.3-Name ohne Pfad
                ; in LongName abgelegt:
		mov si,Offset PathShortName
		mov di,Offset LongName
		call ExtractName

                ; Lnge mit 0 vorbelegen, um spter testen zu knnen,
                ; ob ein plausibler Wert vom Treiber kam:
                mov Word Ptr [Offset EntryNameLen-1],0

		mov ax,ds
		mov si,ax               ; Segment von DirEntry
		mov es,ax               ; Segment von PathShortName
		mov di,Offset DirEntry
		mov bx,Offset PathShortName

		mov ch,0                ; Flag fr Aufbereitung der Rohdaten
		mov ax,150Fh
                ; Die Funktion 150Fh liest den langen Dateinamen aus.
                ; Eingabe: PathShortName (kann verndert werden)
                ; Ausgabe: DirEntry-Struktur (siehe ganz unten)
		int 2Fh
		jnc GetLongOk

		mov Error,102
		jmp GetLongEnd

  GetLongOk:    ; Prfen, ob DirEntry-Struktur um 1 verschoben ist,
                ; wie es bei SHSUCDX der Fall ist:
                mov eax,[LBN_I]
                xchg al,ah
                ror eax,16
                xchg al,ah
                cmp eax,[LBN_M]         ; Intel = Motorola?
                je DirEntryOk

                mov si,Offset EntryName-1
		mov cl,[EntryNameLen-1]
                jmp CheckLength

  DirEntryOk:   mov si,Offset EntryName
		mov cl,[EntryNameLen]

  CheckLength:  cmp cl,0                ; Lnge soll >0 sein
                ja ReadLongName
                mov Error,102
                jmp GetLongEnd

  ReadLongName: ; Langen Dateinamen aus DirEntry-Struktur auslesen
		; und dabei eventuelle Nullbytes auslassen:
		mov di,Offset LongName
                mov ch,0
  GetLongLoop:  lodsb
		cmp al,0
		jz nCopy
                cmp al,';'              ; Dateiname endet mit ";1"
                je SetLongTerm
                stosb
                inc ch
  nCopy:        dec cl
		jnz GetLongLoop

  SetLongTerm:	mov al,'$'              ; Stringterminierung anfgen
		stosb
                mov Error,ch            ; ch = Lnge des Namens

  GetLongEnd:   ret


; ------------------------------------------------------------------------
  PrintInfoZeile:
; ------------------------------------------------------------------------
		call GetQInfo
		mov bx,Offset InfoZeile+6
		call Byte2Str_bis99
		mov al,Min
		mov bx,Offset InfoZeile+11
		call Byte2Str_bis99
		mov al,Sec
		mov bx,Offset InfoZeile+14
		call Byte2Str_bis99
		mov dx,Offset InfoZeile
		mov ah,9
		int 21h
		ret


; ------------------------------------------------------------------------
  GetStatus:
; ------------------------------------------------------------------------
; Rckgabe:     ax = Status  (1=Open 2=Unlocked 2048=NoDisc)

		GetDrive
		call PrepareIOCTLI
		mov [TransAnz],5
		mov Byte Ptr [ControlCode],6
		call DriverRequest
		mov ax,Word Ptr [ControlBlock+1]
		ret
; ------------------------------------------------------------------------
  GetSectorSize:
; ------------------------------------------------------------------------
; Rckgabe:     ax = Sektorgre (CDROM=2048, RAW=2352)

		GetDrive
		call PrepareIOCTLI
		mov [TransAnz],4
		mov Byte Ptr [ControlCode],7
		call DriverRequest
		mov ax,Word Ptr [ControlBlock+2]
		ret
; ------------------------------------------------------------------------
  GetTracks:
; ------------------------------------------------------------------------
; Rckgabe:     FirstTrack und LastTrack

		push ax
		GetDrive
		call PrepareIOCTLI
		mov [TransAnz],7
		mov Byte Ptr [ControlCode],10    ; AudioDiskInfo
		call DriverRequest
		mov ax,Word Ptr [ControlBlock+1]
		mov [FirstTrack],al
		mov [LastTrack],ah
		mov eax,DWord Ptr [ControlBlock+3]
		mov [LeadOut],eax
		pop ax

		ret
; ------------------------------------------------------------------------
  GetTrackStart:
; ------------------------------------------------------------------------
; Ermittelt die Startzeit eines Tracks
; Eingabe:      al  = Track
; Rckgabe:     eax = Startzeit

		GetDrive
		call PrepareIOCTLI
		mov [TransAnz],7
		mov Byte Ptr [ControlCode],11    ; Audio Track Info
		mov Byte Ptr [ControlBlock+1],al ; Track
		call DriverRequest
		mov eax,DWord Ptr [ControlBlock+2]
		ret

; ------------------------------------------------------------------------
  Eject:
; ------------------------------------------------------------------------
		GetDrive
		call PrepareIOCTLO
		mov [TransAnz],1
		mov Byte Ptr [ControlCode],0
		call DriverRequest
		ret
; ------------------------------------------------------------------------
  UnlockDrive:
; ------------------------------------------------------------------------
		GetDrive
		call PrepareIOCTLO
		mov [TransAnz],2
		mov Byte Ptr [ControlCode],1
		mov Byte Ptr [ControlBlock+1],0 ; Unlock
		call DriverRequest
		ret
; ------------------------------------------------------------------------
  LockDrive:
; ------------------------------------------------------------------------
		GetDrive
		call PrepareIOCTLO
		mov [TransAnz],2
		mov Byte Ptr [ControlCode],1
		mov Byte Ptr [ControlBlock+1],1 ; Lock
		call DriverRequest
		ret
; ------------------------------------------------------------------------
  ResetDrive:
; ------------------------------------------------------------------------
		GetDrive
		call PrepareIOCTLO
		mov [TransAnz],1
		mov Byte Ptr [ControlCode],2
		call DriverRequest
		ret
; ------------------------------------------------------------------------
  Close:
; ------------------------------------------------------------------------
		GetDrive
		call PrepareIOCTLO
		mov [TransAnz],1
		mov Byte Ptr [ControlCode],5
		call DriverRequest
		ret


; ------------------------------------------------------------------------
  GetQInfo:
; ------------------------------------------------------------------------
; Fllt die Struktur QInfo mit den aktuellen Werten.
; Der Track im BCD-Format wird gleich konvertiert und steht am Ende in al
; Rckgabe:     al = aktueller Track

		GetDrive
		call PrepareIOCTLI
		mov ax,ds
		mov [TransSeg],ax
		mov [TransOfs],Offset QInfo
		mov [TransAnz],11       ; Lnge von QInfo
		call DriverRequest
                mov al,Track
                mov ah,al               ; BCD -> normal
                shr ah,4
                and al,15
                aad
                mov Track,al
                ret

; ------------------------------------------------------------------------
  GetAudioStatus:
; ------------------------------------------------------------------------
		GetDrive
		call PrepareIOCTLI
		mov ax,ds
		mov [TransSeg],ax
		mov [TransOfs],Offset StatusBlock
		mov [TransAnz],11       ; Lnge der Status-Struktur
		call DriverRequest
                ret


; ------------------------------------------------------------------------
  Seek:
; ------------------------------------------------------------------------
; Eingabe:      eax = Startsektor

		GetDrive
		ClearReqHdr
		mov Byte Ptr [ReqCommand],131
 		mov Byte Ptr [ReqHeader+13],0    ; Addressing Mode "Sektoren"
;		mov Byte Ptr [ReqHeader+13],1    ; Addressing Mode "Red Book" (Min/Sec/Frm)
		mov DWord Ptr [ReqHeader+20],eax ; Start Sektor
		call DriverRequest
		ret

; ------------------------------------------------------------------------
  PlayAudio:
; ------------------------------------------------------------------------
; Startet die Audiowiedergabe
; Eingabe:      eax = Startsektor
;               ebx = Sektorzahl

		GetDrive
		ClearReqHdr
		mov Byte Ptr [ReqCommand],132
		mov Byte Ptr [ReqHeader+13],0    ; Addressing Mode "Sektoren"
		mov DWord Ptr [ReqHeader+14],eax ; Start Sektor
		mov DWord Ptr [ReqHeader+18],ebx ; Sektoranzahl
		call DriverRequest
		ret

; ------------------------------------------------------------------------
  StopAudio:
; ------------------------------------------------------------------------
		GetDrive

		ClearReqHdr
		mov Byte Ptr [ReqCommand],133
		call DriverRequest

		ret

; ------------------------------------------------------------------------
  ResumeAudio:
; ------------------------------------------------------------------------
		GetDrive
		ClearReqHdr
		mov Byte Ptr [ReqCommand],136
		call DriverRequest
		ret

; ------------------------------------------------------------------------
  DriverRequest:
; ------------------------------------------------------------------------
; bergibt mittels der MSCDEX-Funktion 10h einen Request-Header an den
; CDROM-Gertetreiber. Dieser fhrt das darin enthaltene Kommando aus.
; Eingabe:      Ausgefllte Struktur "ReqHeader"
;               cx = Laufwerk (A=0, B=1 ...)

		push eax
		push ebx
                mov ax,ds
		mov es,ax
		mov bx,Offset ReqHeader
		mov ax,1510h            ; CDROM-Treiber ansprechen
		int 2Fh
		mov ax,ReqStatus
		; ReqStatus: 8000h=Error, 200h=Busy, 100h=done, 8=SectorNotFound

		test ax,8000h
		jz ok

                test ax,200h
                jz n_busy
                mov Error,103
                PRINT CDROM-Error=103[Drive_busy]
                NewLine
                jmp ok
  n_busy:       mov Error,102
                PRINT CDROM-Error=102[Driver_request_failed]
                NewLine
                jmp ok

  ok:           pop ebx
		pop eax
		ret

; ------------------------------------------------------------------------
  ExtractName:
; ------------------------------------------------------------------------
; Liefert den Teilstring hinter dem letzten Backslash,
; schneidet also den Pfad ab.
; Eingabe:      si = Offset des Eingabestrings (nullterminiert)
;               di = Offset des Ergebnisstrings ($-terminiert)

		mov bx,di               ; bx = Anfangsposition
		mov ax,ds
		mov es,ax
		cld

  ExtractLoop:  lodsb
		cmp al,'\'
		jne nBackslash
		mov di,bx               ; berschreiben ab 1. Zeichen
		jmp ExtractLoop

  nBackslash:   stosb
		cmp al,0
		jne ExtractLoop
                dec di
                mov al,'$'
                stosb

		ret


; ------------------------------------------------------------------------
  PrepareIOCTLO:
; ------------------------------------------------------------------------
; Bereitet den Request Header fr einen Schreibzugriff zum Treiber vor
		ClearReqHdr
		; Kommando 12 (IOCTLO) einsetzen...
		mov [ReqCommand],12
		; ...und die Transferadresse auf den Control-Block setzen:
		push ax
		mov ax,ds
		mov [TransSeg],ax
		mov [TransOfs],Offset ControlBlock
		pop ax

		ret

; ------------------------------------------------------------------------
  PrepareIOCTLI:
; ------------------------------------------------------------------------
; Bereitet den Request Header fr einen Lesezugriff auf den Treiber vor
		ClearReqHdr

		; Kommando 3 (IOCTLI) einsetzen...
		mov [ReqCommand],3
		; ...und die Transferadresse auf den Control-Block setzen:
		push ax
		mov ax,ds
		mov [TransSeg],ax
		mov [TransOfs],Offset ControlBlock
		pop ax

		ret


; ------------------------------------------------------------------------
  Time2Sector:
; ------------------------------------------------------------------------
; Rechnet eine Zeitangabe in eine Sektornummer um
; Eingabe:      eax = Zeit (---|Min|Sec|Frm)
; Rckgabe:     eax = Sektor

		push bx
		push ecx
		push edx

		xor  edx,edx
		xor  bh,bh
		mov  dl,al                   ; dl=frame
		mov  bl,ah                   ; bl=sec
		shr  eax,16                  ; al=min
		mov  cx,60
		mul  cx
		add  ax,bx
		mov  ecx,75
		mul  ecx
		add  eax,edx
		sub  eax,150

		pop edx
		pop ecx
		pop bx
		ret

; ------------------------------------------------------------------------
  QInfo2Time:
; ------------------------------------------------------------------------
; Extrahiert die aktuelle Gesamtzeit aus der vorher ausgelesenen Struktur
; Eingabe:      GetQInfo mu vorher aufgerufen worden sein
; Rckgabe:     eax = Zeit (---|Min|Sec|Frm)

		xor eax,eax
		mov ah,aMin
		shl eax,8
		mov ah,aSec
		mov al,aFrame
		ret

; ------------------------------------------------------------------------
  FastForward:
; ------------------------------------------------------------------------
; Ermittelt den aktuellen Sektor, erhht ihn und startet die Wiedergabe
		call GetQInfo
		call QInfo2Time
                call Time2Sector
		add  eax,1125           ; 1 Sek. = 75 Sektoren
		call PlayTrackAt
		ret

; ------------------------------------------------------------------------
  FastReverse:
; ------------------------------------------------------------------------
; Ermittelt den aktuellen Sektor, vermindert ihn und startet die Wiedergabe
		call GetQInfo
		call QInfo2Time
                call Time2Sector
		sub  eax,1125           ; 1 Sek. = 75 Sektoren
		call PlayTrackAt
		ret


; ------------------------------------------------------------------------
  SetTrack:
; ------------------------------------------------------------------------
; Bereitet die Wiedergabe eines Tracks vor.
; Falls der angegebene Track nicht zulssig ist, wird er per Wrapping
; korrigiert.
; Eingabe:      al = Track
       
		call GetTracks
		cmp al,LastTrack
		jbe nOver
		mov al,firstTrack
  nOver:        cmp al,firstTrack
		jae inRange
		mov al,LastTrack
  inRange:
		call GetTrackStart
		call Time2Sector
		call Seek
		ret


; ************* Byte2Str_bis99 **********************************
; Eingabe: al=Bytewert 0..99
;          bx=Adresse des Zielstrings
; Rckgabe: 2-stelliger String (00..99)
; vernderte Register: keine
Byte2Str_bis99:
		push ax
		mov ah,'0'
  Hunderter:    cmp al,100
		jb kleiner_100
		sub al,100
		inc ah
		jmp Hunderter
  kleiner_100:  mov [bx],ah             ; Hunderter in String einsetzen
		aam                     ; spaltet AL in AH/AL auf
		add ax,3030h            ; Ziffer -> ASCII
		xchg al,ah
		mov [bx],ax             ; fr 3 statt 2 Stellen: [bx+1]
		pop ax
		ret


; *************************************************
; *  Hex-Zahlausgabe                              *
; *  Eingabe:         ax = Zahl bei PrintHexWord  *
; *             bzw.  al = Zahl bei PrintHexByte  *
; *************************************************
PrintHexWord:   xchg al,ah
		call PrintHexByte
		xchg al,ah
		call PrintHexByte
		ret

PrintHexByte:   push dx
		mov dl,al
		shr dl,4
		call Ziffer
		mov dl,al
		call Ziffer
		pop dx
		ret

  Ziffer:       and dl,0Fh
		add dl,'0'
		cmp dl,'9'
		jbe Schreibe_Ziffer
		add dl,7
  Schreibe_Ziffer: push ax
		mov ah,2
		int 21h
		pop ax
		ret


PrintHexDWord:  ror eax,16
		call PrintHexWord
		ror eax,16
		call PrintHexWord
		NewLine
		ret


PrintString:    push ax
		mov ah,9
		int 21h
		pop ax
		ret



; ************************************************************************
; ************* Konstanten und initialisierte Variablen                 **
; ************************************************************************

SyntaxStr       DB 'CDROM Control and info utility',10,13
		DB '(c) 1999-04-01 - 2002-06-27 by Klaus Peichl (pei@freenet.de)',10,10,13
                DB 'The program expects one of the following parameters:',10,13
                DB 'RESET    Reset the CDROM device driver',10,13
                DB 'EJECT    Eject disc',10,13
                DB 'CLOSE    Close disc tray',10,13
                DB 'DETECT   Detect CD type (Errorlevel: 1=noDisc, 2=Audio, 3=Data)',10,13
                DB 'INFO     Print track number and current time within track',10,13
                DB 'STATUS   Playback status (Errorlevel: 1=Pause, 2=Finished)',10,13
                DB 'GETTRACK Return number of current track as Errorlevel',10,13
                DB 'GETFIRST Return number of first track as Errorlevel',10,13
                DB 'GETLAST  Return number of last track as Errorlevel',10,13
                DB 'PLAY [#] Plays audio track [number #]',10,13
                DB 'PLAY +   Fast Forward (+15 Sec.)',10,13
                DB 'PLAY -   Fast Reverse (-15 Sec.)',10,13
                DB 'STOP     Pauses playback, stops when in Pause mode.',10,13
                DB 'CONTINUE Resumes audio playback from Pause mode',10,13
                DB 'TRACK #  Set audio track number',10,13
                DB 'TRACK +  Increase audio track number',10,13
                DB 'TRACK -  Decrease audio track number',10,13
                DB 'L <path> Print ISO 9660 Level 2 long filename (up to 32 characters)',10,13
                DB '         Returns length of name as ErrorLevel',10,10,13
                DB 'For multiple CDROM drives or disc changer devices, a unit number',10,13
                DB 'may be specified:  CDROM /U# <Command>  (select unit #, default=0)',10,10,13
                DB 'Abnormal return values (ErrorLevel):',10,13
                DB '100 = no drive  101 = Device Error  102 = Driver request failed  103 = Busy'
CR              DB 10,13,'$'
InfoZeile       DB 'Track 00   00:00$'
Error           DB 0
Unit            DB 0

; ************************************************************************
; ************* Datenstrukturen                                         **
; ************************************************************************

ReqHeader       DB 13                   ; Lnge des Request Headers
		DB 0                    ; Subunit
  ReqCommand    DB ?
  ReqStatus     DW ?
		DB 8 Dup (0)            ; reserved

		DB 0                    ; Media descriptor byte from BPB
  TransOfs      DW ?                    ; Ofs(Transfer address)
  TransSeg      DW ?                    ; Seg(Transfer address)
  TransAnz      DW ?                    ; Number of bytes to transfer
		DW 0                    ; Starting sector number
		DD 0                    ; DWORD ptr to requested vol
					; ID if error 0FH
ControlBlock:
ControlCode:    DB ?
		DB 20 Dup (?)           ; diverse Parameter,
					; z.B. StartFrame


QInfo           DB 12                   ; Control block code
		DB ?                    ; CONTROL and ADR byte
  Track         DB ?                    ; Track number (TNO)
		DB ?                    ; (POINT) or Index (X)
		; Running time within a track
  Min           DB ?                    ; (MIN)
  Sec           DB ?                    ; (SEC)
  Frame         DB ?                    ; (FRAME)
	        DB ?                    ; (ZERO)
		; Running time on the disk
  aMin          DB ?                    ; (AMIN) or (PMIN)
  aSec          DB ?                    ; (ASEC) or (PSEC)
  aFrame        DB ?                    ; (AFRAME) or (PFRAME)


StatusBlock     DB 15                   ; Control block code
  AudioStatus   DW ?                    ;
  StartSektor   DD ?                    ; Startsektor
  EndSektor     DD ?                    ; Endsektor


;DirEntry        DB ?                    ; XAR length
;                DD ?                    ; LBN
;	         DW ?                    ; Disk-Groesse
;  EntrySize     DD ?                    ; File-Groesse
;	         DB 7 Dup (?)            ; Datum+Zeit
;	         DB ?                    ; File-Flags
;	         DB ?,?                  ; Interleave
;	         DW ?                    ; Volume Set Sequence Number
;  EntryNameLen  DB ?                    ; Laenge des Dateinamens
;  EntryName     DB 38 Dup (?)           ; Dateiname (nullterminiert)
;	         DW ?                    ; File version
;	         DB ?                    ; Lnge der System Use Daten
;	         DB 220 Dup (?)          ; System Use Daten

DirEntry        DB ?                    ; lenght of DirEntry
                DB ?                    ; XAR length
  LBN_I         DD ?                    ; LBN
  LBN_M         DD ?                    ; LBN
                DD ?                    ; File-Groesse
                DD ?                    ; File-Groesse
                DB 7 Dup (?)            ; Datum+Zeit
                DB ?                    ; File-Flags
                DB ?                    ; Interleave size
                DB ?                    ; Interleave skip factor
                DW ?                    ; Volume Set Sequence Number
                DW ?                    ; Volume Set Sequence Number
  EntryNameLen  DB ?                    ; Laenge des Dateinamens
  EntryName     DB 38 Dup (?)           ; Dateiname (nullterminiert)
                DB 220 Dup (?)          ; System Use Daten


; ************************************************************************
; ************* uninitialisierte Variablen                              **
; ************************************************************************

LongName        DB 255 Dup(?)
PathShortName   DB 255 Dup(?)
FirstTrack      DB ?
LastTrack       DB ?
LeadOut         DD ?

		END Start
