; GetPort liest die Statusleitungen der parallelen Schnittstelle ein
; und liefert in Errorlevel ein Bitmuster (0..31), das den
; gedrckten Tasten entspricht (5 Tasten).

; (C) 1998-09-01 by Klaus Peichl, stliche Ringstr. 7, 85113 Bhmfeld

;LCD_LOW_ACTIVE  EQU 1

IFDEF LCD_LOW_ACTIVE
  PIN14ON       = 2
  PIN14OFF      = 0
ELSE
  PIN14ON       = 0
  PIN14OFF      = 2
ENDIF
; Wenn man die Werte von PIN14ON und PIN14OFF vertauscht, und das
; LCD-Modul mit einem LOW-Pegel einschaltet, verhindert man, da
; beim Booten der uninitialisierte Zustand des LCDs zu sehen ist.

                .MODEL tiny
                .Code
                ORG 100h

Start:          mov dx,Offset Kopf_String
                mov ah,9
                int 21h


; ----------------------------------------------------------------------
  Parameter_Auswertung:
; ----------------------------------------------------------------------
                mov si,81h
  Hole_Zeichen: lodsb
                cmp al,' '
                je Hole_Zeichen
                cmp al,13
                je Main
                cmp al,'/'
                je Schraegstrich
                jmp Hole_Zeichen

  Schraegstrich:
                lodsb
		and al,255-32           ; Klein -> Grobuchstabe

                cmp al,'L'              ; Parameter /LPTx
                jne kein_L
                lodsw                   ; 'PT' berlesen
                lodsb
                sub al,'1'
                cmp al,3
                ja Syntaxfehler
                mov [LPT],al
                jmp Hole_Zeichen
    kein_L:     cmp al,'M'              ; Parameter /M (Mirror)
                jne kein_M
                inc Spiegel
                jmp Hole_Zeichen
    kein_M:     cmp al,'R'              ; Parameter /R (Range)
                jne kein_R
                call ReadParamNumber
                mov Min,al
                call ReadParamNumber
                mov Max,al
                jmp Hole_Zeichen
    kein_R:     cmp al,'K'              ; Parameter /K (Keyboard)
                jne kein_K
                inc UseKbd
                jmp Hole_Zeichen
    kein_K:
                jmp Syntaxfehler

  Syntaxfehler: mov ah,9
                mov dx,Offset Syntax_String
                int 21h
                jmp Programm_entfernen


; ----------------------------------------------------------------------
  Main:
; ----------------------------------------------------------------------

                call SetPort

                mov dx,Offset Status_Str
                mov ah,9
                int 21h

WaitRange:      cmp useKbd,0
                je  NoKbd

                call ReadKey
               call printhexbyte
                cmp al,0
                jne CheckRange

  NoKbd:        call ReadPort

  CheckRange:   cmp al,Min
                jb  WaitRange
                cmp al,Max
                ja  WaitRange

Ausgabe:        call PrintHexByte

Programm_entfernen:
                mov ah,4Ch
                int 21h


; *************************************************
; *  Hex-Zahlausgabe                              *
; *  Eingabe:         ax = Zahl bei PrintHexWord  *
; *             bzw.  al = Zahl bei PrintHexByte  *
; *************************************************
PrintHexWord:   xchg al,ah
                call PrintHexByte
                xchg al,ah
                call PrintHexByte
                ret

PrintHexByte:   push dx
                mov dl,al
                shr dl,1
                shr dl,1
                shr dl,1
                shr dl,1
		call Ziffer
		mov dl,al
		call Ziffer
		pop dx
		ret

  Ziffer:       and dl,0Fh
                add dl,'0'
		cmp dl,'9'
		jbe Schreibe_Ziffer
                add dl,7
  Schreibe_Ziffer: push ax
		mov ah,2
		int 21h
		pop ax
		ret

; ----------------------------------------------------------------------
  ReadKey:
; ----------------------------------------------------------------------

                ; Testen, ob was im Tastaturpuffer ist:
                mov ax,0040h
                mov es,ax
                mov ax,ES:[001Ah]
                cmp ax,ES:[001Ch]
                jne KeyOn
                mov al,0
                jmp KeyEnd

                ; Taste im Puffer lesen:
  KeyOn:        mov ah,7
                int 21h
                cmp al,'1'
                jne n_1
                mov al,8
  n_1:          cmp al,'2'
                jne n_2
                mov al,4
  n_2:          cmp al,'3'
                jne n_3
                mov al,2
  n_3:          cmp al,'4'
                jne n_4
                mov al,1
  n_4:          cmp al,27
                jne n_Esc
                mov al,16
  n_Esc:

KeyEnd:         ret


; ----------------------------------------------------------------------
  ReadPort:
; ----------------------------------------------------------------------
; Rckgabe:     al = Code der gedrckten Taste(n)
; Verndert:    ah

                push dx

;                mov dx,DatenPort
;                mov al,80h
;                out dx,al               ; Bit 7 = 5V (fuer Joystickadapter)

                mov dx,SteuerPort
                mov al,PIN14ON          ; Bit 1 mu 0 sein, damit Pin14=5V
                out dx,al
                mov dx,StatusPort
                in  al,dx
                and al,11111000b        ; Statusleitungen extrahieren
                xor al,01111000b        ; Pegel invertieren (auer Busy)

                mov ah,0
                cmp Spiegel,ah
                jz  default

                ; gespiegeltes (altes) Mapping:
                test al,64
                jz   n_Taste1m
                or   ah,8         ;mov ah,4 (linke Taste)
     n_Taste1m: test al,128
                jz   n_Taste2m
                or   ah,4         ;mov ah,3
     n_Taste2m: test al,32
                jz   n_Taste3m
                or   ah,2         ;mov ah,2
     n_Taste3m: test al,16
                jz   n_Taste4m
                or   ah,1         ;mov ah,1
     n_Taste4m: test al,8
                jz   n_Taste5m
                or   ah,16        ;mov ah,5 (Men-Taste)
     n_Taste5m: jmp  ReadEnd

Default:        test al,64
                jz   n_Taste1
                or   ah,1         ;mov ah,4 (linke Taste)
     n_Taste1:  test al,128
                jz   n_Taste2
                or   ah,2         ;mov ah,3
     n_Taste2:  test al,32
                jz   n_Taste3
                or   ah,4         ;mov ah,2
     n_Taste3:  test al,16
                jz   n_Taste4
                or   ah,8         ;mov ah,1
     n_Taste4:  test al,8
                jz   n_Taste5
                or   ah,16        ;mov ah,5 (Men-Taste)
     n_Taste5:
ReadEnd:        pop dx
                mov al,ah
                ret


; ----------------------------------------------------------------------
  SetPort:
; ----------------------------------------------------------------------
                mov dx,Offset Port_Str
                mov ah,9
                int 21h

                mov bl,[LPT]
                shl bl,1
                mov bh,0
                push es
                mov ax,0040h            ; Adresse der par. Schnittstelle
                mov es,ax               ; ermitteln
                mov ax,ES:[8+bx]
                pop es
                mov DatenPort,ax
                call PrintHexword
                inc ax
                mov StatusPort,ax
                inc ax
                mov SteuerPort,ax
                ret

; ------------------------------------------------------------------------
  ReadParamNumber:
; ------------------------------------------------------------------------
; Liest einen in Ziffern geschriebenen numerischen Wert von der Kommando-
; zeile und wandelt den String in eine Zahl um
; Eingabe:      si = Startindex in der Kommandozeile
; Rckgabe:     ax = Zahl

                push bx
                push cx

                mov cx,10
                xor ax,ax               ; mov ax,0
                xor bx,bx               ; mov bx,0

  Zeichen:      mov bl,[si]
                inc si
                sub bl,'0'              ; ASCII -> Byte-Wert
                cmp bl,'9'
                ja keine_Ziffer
                mul cx                  ; ax mu 10-fach gewichtet werden
                add ax,bx
                jmp Zeichen
  keine_Ziffer:
                pop cx
                pop bx
                ret


; ************************************************************************
; ************* Variablen und Strings                                   **
; ************************************************************************

Datenport       DW ?
Statusport      DW ?
Steuerport      DW ?
LPT             DB 0
Spiegel         DB 0
useKbd          DB 0
Min             DB 0
Max             DB 255
Kopf_String     DB 'GetPort (C) 1998-2000 by Klaus Peichl',10,13,'$'
Port_Str        DB 10,13,'Parallel port (hex): $'
Status_Str      DB 10,13,'Port-Status (hex):   $'
Syntax_String   DB 10,13,'This tool returns the status of 5 keys',10,13
                DB 'connected to a parallel port.',10,13
IFDEF LCD_LOW_ACTIVE
                DB 'Pin 14 of the parallel port is set LOW by this version.',10,13
ENDIF
                DB 10
                DB 'Syntax:     GetPort [/M] [/LPT#] [/R<Min>-<Max>]',10,13
                DB 'Options:    /LPT#  Select parallel port number',10,13
                DB '            /K     Check normal PC-Keyboard as well (keys: 1,2,3,4,Esc)',10,13
                DB '            /M     Use mirrored mapping for values 1,2,4,8',10,13
                DB '            /R     Wait until the status is in the specified range',10,13
                DB '            /?     This help screen',10,13
                DB 'Result:     ErrorLevel=0..31',10,13
                DB 'Example:    GetPort /LPT2 /R1-31',10,13,'$'

                END Start

