; KeyTSR schreibt Scancodes in den Tastaturpuffer, wenn Tasten
; an den Statusleitungen des Parallelports gedrckt werden.
; (C) 15.05.1998 by Klaus Peichl, stliche Ringstr. 7, 85113 Bhmfeld
;
;
; Programmierung als TSR:
; .COM-Programm => DS=CS=ES
;
; -  Sprung in den Installationsteil
;
; -  Kommandozeilenparameter lesen:
;    in DS:[80h] (also im PSP) steht die Lnge des Parameterstrings
;    der String selbst direkt dahinter (mit Leerzeichen)
;
; -  alten Interruptvektor lesen:  ah=35h  al=Nr. INT 21h
;                                  Ergebnis in ES:BX
;
; -  Ergebnis im residenten Teil bis zur Deinstallation aufbewahren
;
; -  Residenten Programmteil mglichst nah an den Segmentanfang
;    kopieren (ab 30h geht's).
;
; -  Vektor auf den Beginn des verschobenen Programmteils setzen:
;                                  Startadresse in DS:DX
;                                  ah=25h  al=Nr. INT 21h
;
; -  Programm resident beenden:    DX=Anzahl der Paragraphen  16 Byte,
;                                     die ab Segmentanfang erhalten bleiben
;                                  ah=31h  al=Returncode  INT 21h
;
; -  Im residenten Teil: Speicherzugriffe nur bzgl. Codesegment machen!


;LCD_LOW_ACTIVE  EQU 1

IFDEF LCD_LOW_ACTIVE
  PIN14ON       = 2
  PIN14OFF      = 0
ELSE
  PIN14ON       = 0
  PIN14OFF      = 2
ENDIF
; Wenn man die Werte von PIN14ON und PIN14OFF vertauscht, und das
; LCD-Modul mit einem LOW-Pegel einschaltet, verhindert man, da
; beim Booten der uninitialisierte Zustand des LCDs zu sehen ist.

                .MODEL tiny
                .Code
                ORG 100h

Start:          jmp Installation


; ----------------------------------------------------------------------
  Neuer_Vektor:
; ----------------------------------------------------------------------
                push ds
                push ax                 ; Register retten
                push bx
                push dx
                push es
                pushf

                push cs
                pop  ds

;                mov dx,DatenPort
;                mov al,80h
;                out dx,al               ; Bit 7 = 5V (fuer Joystickadapter)
                mov dx,SteuerPort
                mov al,PIN14ON          ; Bit 1 mu 0 sein, damit Pin14=5V
                out dx,al

                mov dx,StatusPort
                in  al,dx
                and al,11111000b        ; Statusleitungen extrahieren
                xor al,01111000b        ; Pegel auer Busy invertieren
                or  al,al
                jnz Taste_gedrueckt
                mov bereit,1
                jmp Return

Taste_gedrueckt:
                cmp bereit,1
                jne Return

                cmp Spiegel,0
                jz  default

                ; gespiegeltes (altes) Mapping:
                cmp al,64
                jne n_Taste1m
                mov bl,0                ; linke Taste
     n_Taste1m: cmp al,128
                jne n_Taste2m
                mov bl,2
     n_Taste2m: cmp al,32
                jne n_Taste3m
                mov bl,4
     n_Taste3m: cmp al,16
                jne n_Taste4m
                mov bl,6
     n_Taste4m: cmp al,8
                jne n_Taste5m
                mov bl,8                ; Men-Taste
     n_Taste5m: jmp FetchScanCode

Default:        cmp al,64
                jne n_Taste1
                mov bl,6                ; rechte Taste
     n_Taste1:  cmp al,128
                jne n_Taste2
                mov bl,4
     n_Taste2:  cmp al,32
                jne n_Taste3
                mov bl,2
     n_Taste3:  cmp al,16
                jne n_Taste4
                mov bl,0
     n_Taste4:  cmp al,8
                jne n_Taste5
                mov bl,8                ; Men-Taste
     n_Taste5:

FetchScanCode:  mov bh,0
                mov si,[TabPos]
                mov dx,[si+bx]
                shr bx,1
                mov al,[KeyRep+bx]      ; Autorepeat fr diese Taste?
                mov bereit,al

StoreScanCode:  mov ax,40h
                mov es,ax
                mov bx,ES:[001ch]       ; Buffer Head
                mov ES:[bx],dx          ; Scancode speichern
                inc bx
                inc bx
                cmp bx,001Eh+32
                jb ok
                mov bx,001Eh
            ok:
;                mov ES:[bx],dx
                mov ES:[001ch],bx

Return:         popf
                pop es
                pop dx
                pop bx
                pop ax
                pop ds

                DB 0EAh                 ; EA=JMP
Ofsalt          DW  0227h
Segalt          DW 0FF9Fh


KeySet1         DW 1970h                ; P
                DW 4BE0h                ; Cursor links
                DW 4DE0h                ; Cursor rechts
                DW 011Bh                ; Escape
                DW 011Bh                ; Escape

KeySet2         DW 260Ch                ; Ctrl L
                DW 4BE0h                ; Cursor links
                DW 4DE0h                ; Cursor rechts
                DW 2D18h                ; Ctrl X
                DW 2D18h                ; Ctrl X

KeySet3         DW 1970h                ; P
                DW 095Bh                ; [
                DW 0A5Dh                ; ]
                DW 011Bh                ; Escape
                DW 011Bh                ; Escape

;               Weitere Tasten:
;               DW 1C0Dh                ; Enter


; **********************************************************************
; ************* Variablen (resident gespeichert)                      **
; **********************************************************************

TabPos          DW Offset KeySet1
Datenport       DW ?
Statusport      DW ?
Steuerport      DW ?
bereit          DB 0
KeyRep          DB 0,1,1,0,0            ; Autorepeat fr << und >>
Spiegel         DB 0


; ----------------------------------------------------------------------
  Installation:
; ----------------------------------------------------------------------

Parameter_Auswertung:
                mov si,81h
  Hole_Zeichen: lodsb
                cmp al,' '
                je Hole_Zeichen
                cmp al,13
                ja Parameter
                jmp Residiere
  Parameter:    cmp al,'/'
                je Schraegstrich
                jmp Syntaxfehler

  Schraegstrich:
                lodsb
                and al,0DFh             ; Klein->Grobuchstabe

                cmp al,'L'              ; Parameter /LPTx
                jne kein_L
                lodsw                   ; 'PT' berlesen
                lodsb
                sub al,'1'
                cmp al,3
                ja Syntaxfehler
                mov [LPT],al
                jmp Hole_Zeichen
    kein_L:     cmp al,'M'              ; Parameter /M (Mirror)
                jne kein_M
                inc Spiegel
                jmp Hole_Zeichen
    kein_M:     cmp al,'Q'              ; Parameter /Q (Quiet)
                jne kein_Q
                inc Quiet
                jmp Hole_Zeichen
    kein_Q:     cmp al,'S'              ; Parameter /SETx
                jne kein_S
                lodsw                   ; 'ET' berlesen
                lodsb
                cmp al,'1'
                jne n_Set1
                mov [TabPos],Offset KeySet1
      n_Set1:   cmp al,'2'
                jne n_Set2
                mov [TabPos],Offset KeySet2
      n_Set2:   cmp al,'3'
                jne n_Set3
                mov [TabPos],Offset KeySet3
      n_Set3:   cmp al,':'
                jne n_User
                call UserSet
      n_User:   jmp Hole_Zeichen
    kein_S:     cmp al,'R'              ; /R (remove)
                jne kein_R
                call deinstall
                jmp Programm_entfernen
    kein_R:     cmp al,'P'              ; /P (Probe for Scancodes)
                jne kein_P
                call Probe
                jmp Programm_entfernen
    kein_P:     cmp al,'A'              ; /A (Autorepeat)
                jne kein_A
                mov bx,Offset KeyRep
                mov cx,5
      KeyRepL:  lodsb
                sub al,'0'
                mov Byte Ptr [bx],al
                inc bx
                loop KeyRepL
                jmp Hole_Zeichen

    kein_A:     jmp Syntaxfehler

  Syntaxfehler:
                mov dx,Offset Kopf_String
                call Print
                mov dx,Offset Syntax_String
                call Print
                jmp Programm_entfernen

Residiere:
                mov dx,Offset Kopf_String
                call Print

                call SetPort
                call deinstall          ; falls schon installiert: erst entfernen

                mov dx,Offset Install_Str
                call Print

                mov ax,[DS:2ch]
                mov es,ax
                mov ah,49h
                int 21h                 ; Umgebungsbereich freigeben


                ;cli                    ; Interrupts sperren
                mov ax,251ch            ; Interrupt Nr. 1c umbiegen
                mov dx,Offset neuer_Vektor ; zeigt auf den umkopierten
                int 21h                    ; residenten Teil

                mov dx,31               ; ab Installation freigeben
                                        ; dx=Anzahl der im Speicher zu haltenden
                                        ; Paragraphen  16 Byte
                mov ax,3100h            ; resident beenden
                int 21h


; ----------------------------------------------------------------------
  Deinstall:
; ----------------------------------------------------------------------
                ; bestehenden Int 1Ch-Vektor prfen:
                mov ax,351ch            ; aktuellen Int-Vektor lesen
                int 21h
                mov Segalt,es
                mov Ofsalt,bx
                mov cx,word ptr [ES:bx] ; erstes Wort von aktueller Routine lesen
                cmp cx,word ptr [DS:Neuer_Vektor] ; KEYTSR schon installiert?
                jne nicht_installiert

                push ds
                mov dx,ES:Segalt        ; ursprngl. Segment holen
                mov Segalt,dx           ; als aktuelle Rcksprungadresse
                mov ds,dx
                mov dx,ES:Ofsalt        ; ursprngl. Offset holen
                mov CS:Ofsalt,dx        ; aktuelle Rcksprungadresse
                mov ax,251ch            ; Int.vektor zurcksetzen
                int 21h
                mov ah,49h
                int 21h                 ; bisherige Routine rausschmeissen
                pop ds

                mov dx,Offset Uninstall_Str
                call Print
  nicht_installiert:
                ret

Programm_entfernen:
                mov ax,004ch
                int 21h


; ----------------------------------------------------------------------
  SetPort:
; ----------------------------------------------------------------------
                mov dx,Offset Port_Str
                call Print

                mov bl,[LPT]
                shl bl,1
                mov bh,0
                push es
                mov ax,0040h            ; Adresse der par. Schnittstelle
                mov es,ax               ; ermitteln
                mov ax,ES:[8+bx]
                pop es
                mov DatenPort,ax
                cmp Quiet,0
                jnz w5
                call PrintHexword
  w5:           inc ax
                mov StatusPort,ax
                inc ax
                mov SteuerPort,ax

                mov dx,Offset CR
                call Print
                ret

; ----------------------------------------------------------------------
  UserSet:
; ----------------------------------------------------------------------
                call GetScanCode
                mov [KeySet1],ax
                call GetScanCode
                mov [KeySet1+2],ax
                call GetScanCode
                mov [KeySet1+4],ax
                call GetScanCode
                mov [KeySet1+6],ax

                call GetHex

                mov [KeySet1+8],ax
                ret

GetScanCode:
; Eingabe:      si = aktueller Index in Kommandozeile
; Rckgabe:     ax = angegebener Scancode+ASCII-Wert
                call GetHex
                push ax
                lodsb
                cmp al,','
                je  KommaDa
                mov dx,Offset KeinKommaStr
                call Print
  KommaDa:      pop ax
                ret

; ----------------------------------------------------------------------
  GetHex:
; ----------------------------------------------------------------------
                push bx
                xor bx,bx
                call AddDigit
                call AddDigit
                call AddDigit
                call AddDigit
                mov ax,bx
                pop bx
                ret

  AddDigit:     shl bx,1
                shl bx,1
                shl bx,1
                shl bx,1
                lodsb
                sub al,'0'
                cmp al,9
                jbe Numeric
                and al,255-32           ; Klein- -> Grobuchstabe
                sub al,'A'-10
  Numeric:      and ax,15
                add bx,ax
                ret

; ----------------------------------------------------------------------
  Probe:
; ----------------------------------------------------------------------
                mov ax,40h
                mov es,ax
                mov ax,ES:[001Ch]
                mov bx,ES:[001Ah]       ; Buffer tail
                cmp ax,bx               ; Scancode im Puffer?
                jz  Probe
                mov ax,ES:[bx]          ; Scancode lesen
                inc bx
                inc bx
                cmp bx,001Eh+32
                jb ok2
                mov bx,001Eh
          ok2:  mov ES:[001Ah],bx       ; Tail nachziehen
                call PrintHexWord
                mov dx,Offset CR
                call Print
                cmp ax,011Bh
                jne Probe

                ret


; ----------------------------------------------------------------------
  Print:
; ----------------------------------------------------------------------
; Gibt einen String aus, wenn Quiet nicht 0 ist.
; Eingabe:      dx = Offset des Strings

                cmp Quiet,0
                jnz PrintEnd
                push ax
                mov ah,9
                int 21h
                pop ax
  PrintEnd:     ret


; *************************************************
; *  Hex-Zahlausgabe                              *
; *  Eingabe:         ax = Zahl bei PrintHexWord  *
; *             bzw.  al = Zahl bei PrintHexByte  *
; *************************************************
PrintHexWord:   xchg al,ah
                call PrintHexByte
                xchg al,ah
                call PrintHexByte
                ret

PrintHexByte:   push dx
                mov dl,al
                shr dl,1
                shr dl,1
                shr dl,1
                shr dl,1
		call Ziffer
		mov dl,al
		call Ziffer
		pop dx
		ret

  Ziffer:       and dl,0Fh
                add dl,'0'
		cmp dl,'9'
		jbe Schreibe_Ziffer
                add dl,7
  Schreibe_Ziffer: push ax
		mov ah,2
		int 21h
		pop ax
		ret


; **********************************************************************
; ************* Variablen und Strings (nicht resident)                **
; **********************************************************************


LPT             DB 0
Quiet           DB 0
Kopf_String     DB 'KeyTSR keyboard emulator, (C) 1998-2000 by Klaus Peichl',10,13
IFDEF LCD_LOW_ACTIVE
                DB 'Pin 14 of the parallel port is set LOW by this version.',10,13
ENDIF
                DB 'Type "KEYTSR /?" for help.',10
CR              DB 10,13,'$'
Port_Str        DB 'Parallel port: $'
KeinKommaStr    DB '"," expected!',10,13,'$'
Syntax_String   DB 'Syntax:     KeyTSR [options]',10,10,13
                DB 'Options:    /R       Remove KeyTSR from memory',10,13
                DB '            /Q       Quiet: Suppress output',10,13
                DB '            /M       Use mirrored mapping for keys 1,2,3,4',10,13
                DB '            /?       This help page',10,13
                DB '            /LPT#    Use parallel port # (default=1)',10,13
                DB '            /SET#    Use predefined key mapping set # (default=1)',10,13
                DB '            /SET:Code1,Code2,Code3,Code4,Code5  (5 4-digit hex codes)',10,13
                DB '                     Supply scancodes for user defined key mapping',10,13
                DB '            /P       Probe for scancodes (ESC exits)',10,13
                DB '            /A#####  Set autorepeat for each of the 5 keys to 0 or 1',10,13
                DB '                     (Default=01100 so that <- and -> will be repeated)',10,10,13
                DB 'Predefined keysets:     Key1  Key2  Key3  Key4  Menu-key',10,13
                DB '            /SET1         P    <-    ->    Esc    Esc     (for MPXPLAY)',10,13
                DB '            /SET2        ^L    <-    ->    ^X     ^X      (for LapLink)',10,13
                DB '            /SET3         P     [    ]     Esc    Esc     (for DAMP)',10,10,13
                DB 'Example:    KeyTSR /LPT2 /A01100 /SET:1970,4BE0,4DE0,011B,011B',10,13,'$'
Install_Str     DB 'KeyTSR installed.',10,13,'$'
Uninstall_Str   DB 'Resident KeyTSR removed.',10,13,'$'

       END Start

