; **********************************************************************
; ************* LCDECHO Schreibt einen String ins LCD-Display         **
; ************* (C) 1999-05-21 by Klaus Peichl                        **
; *************                   Dornbergstr. 10                     **
; *************                   91058 Erlangen-Tennenlohe           **
; *************                   (pei@freenet.de)                    **
; **********************************************************************

;LCD_LOW_ACTIVE  EQU 1

IFDEF LCD_LOW_ACTIVE
  PIN14ON       = 2
  PIN14OFF      = 0
ELSE
  PIN14ON       = 0
  PIN14OFF      = 2
ENDIF
; Wenn man die Werte von PIN14ON und PIN14OFF vertauscht, und das
; LCD-Modul mit einem LOW-Pegel einschaltet, verhindert man, da
; beim Booten der uninitialisierte Zustand des LCDs zu sehen ist.

		.Model tiny
		.Code
		Org 100h


; ------------------------------------------------------------------------
  OutPort       MACRO S
; ------------------------------------------------------------------------
                mov al,&S
                xor al,isPofo
                out dx,al
		ENDM


; ------------------------------------------------------------------------
  INT21         MACRO
		LOCAL Ok
; ------------------------------------------------------------------------
		int 21h
		jnc Ok
		mov dx,Offset FileErrStr
		mov ah,9
		int 21h
		jmp Ende                ; Achtung: Stack!
	    ok: ENDM


; ------------------------------------------------------------------------
  GetStabe      MACRO
; ------------------------------------------------------------------------
                lodsb
		and al,255-32           ; Klein -> Grobuchstabe
		ENDM

; **********************************************************************
; ************* Hauptprogramm                                         **
; **********************************************************************

Start:          cmp sp,Offset Stapelend
                ja  StapelOk
                mov dx,Offset NoMemStr
                mov ah,9
                int 21h
                jmp Ende

StapelOk:       call SetPort            ; Default: LPT1

                push ds
		pop es
		cld

		mov si,80h
                lodsb
                cmp al,1
                ja ParamLoop
                jmp SyntaxFehler
ParamLoop:      lodsb
                cmp al,13
                ja  Parameter
                jmp Ende
  Parameter:    cmp al,'/'
                je  Switch
                cmp al,' '
                ja  String
                cmp StringLen,0
		jz  ParamLoop
  String:       inc StringLen
                call LCDchar
                jmp ParamLoop

Switch:         GetStabe
                cmp al,'I'
                jne n_Init

                mov dx,SteuerPort
                OutPort PIN14OFF        ; Spannung an Pin 14 aus
                call WaitLong
                OutPort PIN14ON         ; Spannung an Pin 14 ein
                mov cx,20
  WaitOn:       call WaitLong
                loop WaitOn

                mov al,38h              ; 8Bit, 2Zeilen, 8x7-Matrix
                call LCDcommand
                mov al,8+4              ; Display ON, Cursor OFF
                call LCDcommand
                mov al,6                ; Cursor wandert nach rechts
                call LCDcommand
                mov al,1                ; Clear&Home
                call LCDcommand
                call WaitLong


mov al,56+4     ; RE-Bit setzen
call LCDcommand
call WaitLong
mov al,8+1
call LCDcommand
call WaitLong
mov al,56+0     ; RE-Bit lschen
call LCDcommand
call WaitLong



                jmp ParamLoop

  n_Init:       cmp al,'L'
                jne n_SetLPT

                lodsw                   ; 'PT' berlesen
                lodsb
                sub al,'1'
                cmp al,3
                mov [LPT],al
                ja  WrongPort
                call SetPort
                jmp ParamLoop
    WrongPort:  jmp Syntaxfehler

  n_SetLPT:     cmp al,'A'
                jne n_SetAdd
                lodsw                   ; 'dr' berlesen
                call ReadParamNumber
                or al,128
                mov Adresse,al          ; Adresse fr Scroll-Mode aufheben
                call LCDcommand
                jmp ParamLoop

  n_SetAdd:     cmp al,'W'
                jne n_SetWait
                call ReadParamNumber
                mov Wartezeit,ax
                jmp ParamLoop

  n_SetWait:    cmp al,'F'              ; <F>ile?
                jne n_ReadFile
                call ReadFile
                mov dx,Offset Puffer
                call LCDstring
                jmp ParamLoop

  n_ReadFile:   cmp al,'T'              ; <T>SR?
                jne n_TSR
                call ReadFile
                call PreProcess
                jmp  Install

  n_TSR:        cmp al,'R'              ; <R>emove?
                jne n_Remove
                call DeInstall
                jmp ParamLoop

  n_Remove:     cmp al,'H'
                jne n_Home

  n_Home:       cmp al,'S'              ; <S>croll?
                jne n_Scroll
                lodsb
                cmp al,'+'
                je  ScrollR
                cmp al,'-'
                je  ScrollL
                call ReadFile
                call ScrollUntilKey
                jmp ParamLoop
  ScrollL:      mov al,16+8             ; Scroll L
                jmp Scroll
  ScrollR:      mov al,16+8+4           ; Scroll R
  Scroll:       call LCDcommand
                jmp ParamLoop

  n_Scroll:     cmp al,'U'
                jne n_User
                mov bx,Offset Unterlaengen
                call UserChars
                jmp ParamLoop

  n_User:       cmp al,'M'
                jne n_Mapping
                mov Mapping,1
                jmp ParamLoop
  n_Mapping:    cmp al,'C'              ; "C"lock?
                jne n_Clock
                mov ah,2ch
                int 21h                 ; Timer lesen
                mov ax,cx
                call LCDbigDigits
                jmp ParamLoop
  n_Clock:
Syntaxfehler:   mov dx,Offset SyntaxStr
                mov ah,9
                int 21h
Ende:           mov ax,4C00h
                int 21h


; ------------------------------------------------------------------------
  LCDcommand:
; ------------------------------------------------------------------------
                push ax
                push dx
                mov dx,DatenPort
                out dx,al
                mov dx,SteuerPort
                OutPort 8 + PIN14ON
                call WaitShort
                OutPort 0 + PIN14ON
                call WaitShort
                pop dx
                pop ax
                ret

; ------------------------------------------------------------------------
  LCDchar:
; ------------------------------------------------------------------------
; Eingabe:      al = Zeichen

                push ax
                push dx
                cmp Mapping,1
                jne no_Mapping
                cmp al,'g'
                jne n_g
                mov al,0
  n_g:
                cmp al,'j'
                jne n_j
                mov al,1
  n_j:
                cmp al,'p'
                jne n_p
                mov al,2
  n_p:
                cmp al,'q'
                jne n_q
                mov al,3
  n_q:
                cmp al,'y'
                jne n_y
                mov al,4
  n_y:
  no_Mapping:   mov dx,DatenPort
                out dx,al
                ; Falls Datenport langsam, eventuell hier warten
                mov dx,SteuerPort
                OutPort 4  +PIN14ON     ; Register Select ein
                OutPort 8+4+PIN14ON     ; Enable ein
                call WaitShort
                OutPort 4  +PIN14ON     ; Enable aus
                OutPort 0  +PIN14ON     ; Register Select aus
                call WaitShort
                pop dx
                pop ax
                ret

; ------------------------------------------------------------------------
  LCDstring:
; ------------------------------------------------------------------------
; Eingabe:      dx = Offset des Strings, Endmarkierung: 255

                push si
                mov si,dx
StringLoop:     lodsb
                cmp al,255
                je StringEnd
                cmp al,10
                je StringLoop
                cmp al,13               ; Carriage Return?
                jne n_CR

                ; Testen, in welcher Zeile die zuletzt gesetzte Adresse
                ; liegt. Danach wird zur Anfangsadresse der nchsten
                ; Zeile vorgerckt.
                ; 80h = Zeile1
                ; 94h = Zeile3
                ; C0h = Zeile2
                ; D4h = Zeile4
                mov al,Adresse
                cmp al,94h
                jae n_Zeile1
                mov al,0C0h              ; Zeile1 -> Zeile2
                jmp CR
  n_Zeile1:     cmp al,0C0h
                jae n_Zeile3
                mov al,0D4h              ; Zeile3 -> Zeile4
                jmp CR
  n_Zeile3:     cmp al,0D4h
                jae n_Zeile2
                mov al,94h              ; Zeile2 -> Zeile3
                jmp CR
  n_Zeile2:     mov al,128              ; Zeile4 -> Zeile1
  CR:           call LCDcommand
                jmp StringLoop

  n_CR:         call LCDchar
                jmp StringLoop
StringEnd:      pop si
                ret


; ------------------------------------------------------------------------
  WaitShort:
; ------------------------------------------------------------------------
                push cx
                mov cx,Wartezeit
  ShortLoop:    or cx,cx
                dec cx
                nop
                push ax
                pop ax
                jnz ShortLoop
                pop cx
                ret

; ------------------------------------------------------------------------
  WaitLong:
; ------------------------------------------------------------------------
                push cx
                mov cx,100
  LongLoop:     call WaitShort
                loop LongLoop
                pop cx
                ret

; ------------------------------------------------------------------------
  UserChars:
; ------------------------------------------------------------------------
; Ldt die 8 benutzerdefinierbaren Zeichen ins LCD
; Eingabe:      bx = Offset der Zeichenbitmaps

                push ax
                push bx
                push cx
                mov al,64               ; Character-RAM-Adr=0
                call LCDcommand
                mov cx,64               ; Anzahl zu bertragender Bytes
  UserLoop:     mov al,[bx]
                call LCDchar
                call WaitShort
                inc bx
                loop UserLoop
                mov al,128              ; Text-RAM-Adr=0
                call LCDcommand
                pop cx
                pop bx
                pop ax
                ret

; ------------------------------------------------------------------------
  LCDbigDigits:
; ------------------------------------------------------------------------
                push bx
                mov bx,Offset SiebenSegment
                call UserChars
                push ax
                mov bl,10
                mov al,ah
                mov ah,0
                div bl
                mov di,Offset BigDigits
                call MakeDigit          ; 1. Ziffer
                mov al,ah
                mov di,Offset BigDigits+4
                call MakeDigit          ; 2. Ziffer
                pop ax
                mov bl,10
                mov ah,0
                div bl
                mov di,Offset BigDigits+9
                call MakeDigit          ; 3. Ziffer
                mov al,ah
                mov di,Offset BigDigits+13
                call MakeDigit          ; 4. Ziffer
                pop bx

                mov al,128
                mov Adresse,al
                call LCDcommand
                mov dx,Offset BigDigits
                call LCDstring

                ret

MakeDigit:
; Fllt fr eine 7-Segment-Ziffer die 6 bentigten Zeichen im LCD-Puffer aus
; Eingabe:      al = Ziffer (0..9)
;               di = Offset fr linkes oberes Zeichen
; verndert:    di
                push ax
                cld
                mov ah,al
                mov bx,Offset oben_links
                xlat
                stosb
                mov bx,Offset oben_mitte
                mov al,ah
                xlat
                stosb
                mov bx,Offset oben_rechts
                mov al,ah
                xlat
                stosb
                add di,14
                mov bx,Offset unten_links
                mov al,ah
                xlat
                stosb
                mov bx,Offset unten_mitte
                mov al,ah
                xlat
                stosb
                mov bx,Offset unten_rechts
                mov al,ah
                xlat
                stosb
                pop ax
		ret



; ------------------------------------------------------------------------
  ScrollUntilKey:
; ------------------------------------------------------------------------
                push si
                mov [Platzhalter],' '
                mov [Platzhalter+1],' '
                mov [Platzhalter+2],' '

ScrollBegin:    mov di,Offset Platzhalter

ScrollText:     mov al,Adresse
                call LCDcommand

                mov si,di
                mov cx,20
                cld
  DispLine:     lodsb
                cmp al,255              ; Am Stringende?
                jne W
                mov si,Offset Puffer-3  ; Zurck zum Platzhalter vor
                lodsb                   ; dem String

  w:            call LCDChar
                loop DispLine

                mov al,50
  ScrollDelay:  call WaitLong
                dec al
                jnz ScrollDelay

                mov dx,StatusPort
                in  al,dx
                and al,11111000b        ; Statusleitungen extrahieren
                xor al,01111000b        ; Pegel auer Busy invertieren
                or  al,al
                jnz Taste

                ; Testen, ob was im Tastaturpuffer ist:
                push es
                mov ax,0040h
                mov es,ax
                mov ax,ES:[001Ah]
                cmp ax,ES:[001Ch]
                pop es
                jne Taste

                inc di
                cmp Byte Ptr [di],255
                jne ScrollText
                jmp ScrollBegin

Taste:          pop si
                ret

; ------------------------------------------------------------------------
  ReadParamNumber:
; ------------------------------------------------------------------------
; Liest einen in Ziffern geschriebenen numerischen Wert von der Kommando-
; zeile und wandelt den String in eine Zahl um
; Eingabe:      si = Startindex in der Kommandozeile
; Rckgabe:     ax = Zahl

                push bx
                push cx

                mov cx,10
                xor ax,ax               ; mov ax,0
                xor bx,bx               ; mov bx,0

  Hole_Zeichen: mov bl,[si]
                inc si
                sub bl,'0'              ; ASCII -> Byte-Wert
                cmp bl,'9'
                ja keine_Ziffer
                mul cx                  ; ax mu 10-fach gewichtet werden
                add ax,bx
                jmp Hole_Zeichen
  keine_Ziffer:
                pop cx
                pop bx
                ret

; ------------------------------------------------------------------------
  SetPort:
; ------------------------------------------------------------------------
                push ax
                push bx
                mov bl,[LPT]
                shl bl,1
                mov bh,0
                push es
                mov ax,0040h            ; Adresse der par. Schnittstelle
                mov es,ax               ; ermitteln
                mov ax,ES:[8+bx]
                pop es
                mov DatenPort,ax
                inc ax
                cmp ax,8079h            ; Portfolio?
                jne n_PF
                mov isPofo,2
                mov SteuerPort,ax       ; Status- und Steuerport
                inc ax                  ; sind beim Pofo vertauscht!
                mov StatusPort,ax
                jmp SetPortEnd
  n_PF:         mov StatusPort,ax
                inc ax
                mov SteuerPort,ax
  SetPortEnd:   pop bx
                pop ax
                ret



; ------------------------------------------------------------------------
  ReadFile:
; ------------------------------------------------------------------------
; Liest die als Parameter angegebene Datei in den Pufferspeicher
; (max. 2000 Zeichen)
; Eingabe:        si = Index in Kommandozeile ab dem nach dem Dateinamen
;                      gesucht wird
; Rckgabe:       si = Index in Kommandozeile hinter dem Dateinamen

                call SkipWhiteSpace
                mov dx,si               ; Anfangspos. Dateiname
                dec dx

                call FindWhiteSpace

		dec si
		mov Byte Ptr [si],0     ; Dateiname-Endmarkierung setzen

		mov ax,3D00h            ; Open File (read)
		INT21

OpenOk:         mov bx,ax               ; bx := Handle
		mov cx,2000
		mov dx,Offset Puffer
		mov ax,3F00h            ; Daten einlesen
		INT21

		mov di,ax               ; Anzahl gelesener Bytes
                mov [Puffer+di],255     ; String-Endmarkierung setzen

		mov ax,3E00h            ; Datei schlieen
		INT21

                ret


; ------------------------------------------------------------------------
  PreProcess:
; ------------------------------------------------------------------------
; Fhrt eine Vorverarbeitung des Scripts durch, das im Hintergrund
; abgearbeitet werden soll.
; Eingabe:  Offset Puffer = Script im Klartext
; Rckgabe: Offset Script = Script tokenized
                cld
                mov si,Offset Puffer
                mov di,Offset Script
PreProcessLoop: call SkipWhiteSpace
		and al,255-32           ; Klein -> Grobuchstabe
                stosb
                mov ah,al
                call FindWhiteSpace
                call SkipWhiteSpace
                dec si
                cmp ah,'A'              ; Adr x
                jne n_A
                call ReadParamNumber
                stosw
                jmp PreProcessLoop
  n_A:          cmp ah,'S'              ; Scr x,y
                jne n_S
                call ReadParamNumber
                shl al,1
                mov bx,ax
                call ReadParamNumber
                mov ah,160
                mul ah
                add ax,bx               ; ax = x*2 + y*160
                stosw
                jmp PreProcessLoop
  n_S:          cmp ah,'C'              ; Chr C
                jne n_C
                movsb
                jmp PreProcessLoop
  n_C:          cmp ah,'F'              ; Find C
                jne n_F
                movsb
                jmp PreProcessLoop
  n_F:          cmp ah,'I'              ; Inc
                je PreProcessLoop
                cmp ah,'D'              ; Dec
                je PreProcessLoop
                cmp ah,'G'              ; Grab
                je PreProcessLoop

                ; alles andere wird als Ende-Marke interpretiert
PreProcessEnd:  cmp ah,'E'
                je ScriptOk
                mov dx,Offset ScriptErrStr
                mov ah,9
                int 21h
  ScriptOk:     mov al,'E'
                stosb
                ret

; ------------------------------------------------------------------------
  Install:
; ------------------------------------------------------------------------
                call DeInstall

                mov ax,251ch            ; Interrupt Nr. 1c umbiegen
                mov dx,Offset NewVector ; zeigt auf den residenten Teil
                int 21h

                mov ax,[DS:002Ch]
                mov es,ax
                mov ah,49h
                int 21h                 ; Umgebungsbereich freigeben

                mov dx,Offset Platzhalter
                shr dx,1
                shr dx,1
                shr dx,1                ; dx=Anzahl der im Speicher zu haltenden
                shr dx,1                ; Paragraphen  16 Byte
                inc dx

                mov ax,3100h            ; resident beenden
                int 21h

; ----------------------------------------------------------------------
  DeInstall:
; ----------------------------------------------------------------------
                ; bestehenden Int 1Ch-Vektor prfen:
                mov ax,351ch            ; aktuellen Int-Vektor lesen
                int 21h
                mov Segalt,es
                mov Ofsalt,bx
                mov cx,word ptr [ES:bx] ; erstes Wort von aktueller Routine lesen
                cmp cx,word ptr [DS:NewVector] ; LCDECHO schon installiert?
                jne nicht_installiert

                push ds
                mov dx,ES:Segalt        ; ursprngl. Segment holen
                mov Segalt,dx           ; als aktuelle Rcksprungadresse
                mov ds,dx
                mov dx,ES:Ofsalt        ; ursprngl. Offset holen
                mov CS:Ofsalt,dx        ; aktuelle Rcksprungadresse
                mov ax,251ch            ; Int.vektor zurcksetzen
                int 21h
                mov ah,49h
                int 21h                 ; bisherige Routine rausschmeissen
                pop ds

                mov dx,[SteuerPort]     ; Pegel zurcksetzen
                OutPort 0 + PIN14ON

                mov dx,Offset Uninstall_Str
                mov ah,9
                int 21h
  nicht_installiert:
                ret


; ------------------------------------------------------------------------
  NewVector:
; ------------------------------------------------------------------------
                push es
                push si
                push ax                 ; Register retten
                push bx
                push cx
                push dx
                push ds
                pushf

                push cs
                pop  ds
                mov si,[CS:ScriptPointer]
                cmp Byte Ptr [CS:ScriptPhase],0
                jz ReadScript
                jmp ScriptPhase2

  ReadScript:   mov al,[CS:si]
                inc si
                cmp al,'A'
                jne n_Script_A
                mov al,[CS:si]
                inc si
                or  al,128              ; LCD-Adr. setzen
                mov dx,CS:DatenPort
                out dx,al
                mov dx,CS:SteuerPort
                OutPort 8 + PIN14ON
                jmp PhaseEnd
  n_Script_A:   cmp al,'S'              ; Screen-Position setzen
                jne n_Script_S
                mov bx,[CS:si]
                inc si
                inc si
                mov [CS:ScreenPos],bx
                jmp ReadScript
  n_Script_S:   cmp al,'G'              ; Grab (Zeichen bertragen)
                jne n_Script_G
                mov ax,0B800h           ; Bildspeicher
                mov es,ax
                mov bx,[CS:ScreenPos]
                mov al,[ES:bx]
                jmp ScriptChar
  n_Script_G:   cmp al,'C'              ; Chr C
                jne n_Script_C
                mov al,[CS:si]
                inc si
  ScriptChar:   mov dx,CS:DatenPort
                out dx,al
                mov dx,CS:SteuerPort
                OutPort 4+PIN14ON
                OutPort 4+8+PIN14ON
                jmp PhaseEnd
  n_Script_C:   cmp al,'I'              ; Inc
                jne n_Script_I
                inc ScreenPos
                inc ScreenPos
                jmp ReadScript
  n_Script_I:   cmp al,'D'              ; Dec
                jne n_Script_D
                dec ScreenPos
                dec ScreenPos
                jmp ReadScript
  n_Script_D:   cmp al,'F'              ; Find
                jne n_Script_F
                mov ax,0B800h           ; Bildspeicher
                mov es,ax
                mov ah,[CS:si]          ; ah=Sollwert
                inc si

                mov bx,[CS:ScreenPos]
                mov cx,2000             ; maximal 2000 Zeichen durchsuchen
    FindLoop:   mov al,[ES:bx]
                cmp al,ah
                je  gefunden
                inc bx
                loop FindLoop
                jmp ReadScript          ; Suche fehlgeschlagen
    gefunden:   mov [CS:ScreenPos],bx
                jmp ReadScript

  n_Script_F:   cmp al,'E'
                jne n_Script_E
                mov si,Offset Script
                jmp ReadScript
  n_Script_E:   jmp PhaseEnd


ScriptPhase2:   mov dx,CS:SteuerPort
                OutPort 0 + PIN14ON

PhaseEnd:       mov [CS:ScriptPointer],si
                xor Byte Ptr [CS:ScriptPhase],1

Return:         popf
                pop ds
                pop dx
                pop cx
                pop bx
                pop ax
                pop si
                pop es

                DB 0EAh                 ; EA=JMP
Ofsalt          DW  0227h
Segalt          DW 0FF9Fh



; *************************************************
; *  Hex-Zahlausgabe                              *
; *  Eingabe:         ax = Zahl bei PrintHexWord  *
; *             bzw.  al = Zahl bei PrintHexByte  *
; *************************************************
PrintHexWord:   xchg al,ah
		call PrintHexByte
		xchg al,ah
		call PrintHexByte
		ret

PrintHexByte:   push dx
		mov dl,al
		shr dl,1
		shr dl,1
		shr dl,1
		shr dl,1
		call Ziffer
		mov dl,al
		call Ziffer
		pop dx
		ret

  Ziffer:       and dl,0Fh
		add dl,'0'
		cmp dl,'9'
		jbe Schreibe_Ziffer
		add dl,7
  Schreibe_Ziffer: push ax
		mov ah,2
		int 21h
		pop ax
		ret


; ------------------------------------------------------------------------
  SkipWhiteSpace:
; ------------------------------------------------------------------------
                lodsb
		cmp al,' '
		jbe SkipWhiteSpace
                ret

; ------------------------------------------------------------------------
  FindWhiteSpace:
; ------------------------------------------------------------------------
                lodsb
		cmp al,' '
		ja FindWhiteSpace
                ret

; ************************************************************************
; ************* Konstanten und Variablen                                **
; ************************************************************************

SyntaxStr       DB 'LCDECHO  (c) 1999-05-15 / 2000-06-21 by Klaus Peichl',10,13
                DB 'This tool controls HD44780-compatible LCDs connected to a parallel port.',10,13
IFDEF LCD_LOW_ACTIVE
                DB 'Pin 14 of the parallel port is set LOW by this version.',10,13
ENDIF
                DB 10
		DB 'Syntax:  LCDECHO [Options] <String>',10,10,13
                DB 'Use any of the following options in the below order:',10,10,13
                DB ' /LPT#  Set parallel port # (1..4, default=1)',10,13
                DB ' /W#    Set internal waitloop (#=1..65535, default=1000)',10,13
                DB ' /I     Initialize display',10,13
                DB ' /Adr#  Begin at address #  (0..127, default=current)',10,13
                DB ' /U     Download (nicer) user defined characters "gjpqy"',10,13
                DB ' /M     Map "gjpqy" to user defined characters',10,13
                DB ' /C     Display a big digital clock',10,13
                DB ' /S-    Scroll text left by 1 character',10,13
                DB ' /S+    Scroll text right by 1 character',10,13
                DB ' /S     Scroll content of file "<String>" while keypad is idle',10,13
                DB ' /F     Display content of file "<String>"',10,13
                DB ' /T     Load and activate TSR-script "<string>"',10,13
                DB ' /R     Stop TSR-script and remove resident portion from memory',10,13,'$'
NoMemStr        DB '?OUT OF MEMORY  ERROR',10,13,'$'
FileErrStr      DB 'LCDECHO Fatal: File I/O Error!',10,13,'$'
ScriptErrStr    DB 'LCDECHO: TSR-script error!',10,13,'$'
Uninstall_Str   DB 'Resident LCDECHO removed.',10,13,'$'

; Bitmaps fr benutzerdefinierte Zeichen:

Unterlaengen    DB 0,0,15,17,17,15, 1,14  ; g
                DB 2,0, 6, 2, 2, 2,18,12  ; j
                DB 0,0,30,17,17,30,16,16  ; p
                DB 0,0,13,19,17,15, 1, 1  ; q
                DB 0,0,17,17,17,15, 1,14  ; y

SiebenSegment   DB  0, 0,0,0,0,0,31,31    ; _
                DB  1, 3,3,3,3,3, 3, 1    ;  |
                DB 31,31,0,0,0,0,14,31    ; =
                DB 30,29,3,3,3,3,29,30    ; ]
                DB  3, 3,3,3,3,3,29,30    ; _|
                DB 30,29,3,3,3,3, 3, 2    ; 7
                DB 31,31,0,0,0,0, 0, 0    ; -

; Zusammensetzung der 7-Segment-Ziffern aus je 6 Zeichen:
;                  0  1  2  3  4  5  6  7  8  9
oben_links      DB 1,32,32,32, 1, 1, 1,32, 1, 1
oben_mitte      DB 6,32, 2, 2, 0, 2, 2, 6, 2, 2
oben_rechts     DB 5, 1, 3, 3, 4, 2, 2, 5, 3, 3
unten_links     DB 1,32, 1,32,32,32, 1,32, 1,32
unten_mitte     DB 0,32, 0, 0,32, 0, 0,32, 0, 0
unten_rechts    DB 4, 1, 0, 4, 1, 4, 4, 1, 4, 4

BigDigits       DB 8 Dup(32),0, 7 Dup(32),13
                DB 8 Dup(32),0, 7 Dup(32),255

Mapping         DB 0                    ; Unterlngen verwenden?
isPofo          DB 0
LPT             DB 0
StringLen       DB 0
Wartezeit       DW 1000
ScriptPointer   DW Offset Script
ScriptPhase     DB 0
Adresse         DB 128
Datenport       DW ?
Statusport      DW ?
Steuerport      DW ?
ScreenPos       DW ?
Script          DB 400 Dup (?)
Platzhalter     DB 3 Dup(?) ;           ; Abstand beim Scrollen
Puffer          DB 2002 Dup (?)
Stapel          DB 500 Dup (?)
StapelEnd       DB ?
		END Start
