; **********************************************************************
; ************* SHOWTAG Gibt den MP3-Tag einer Datei aus              **
; ************* (C) 1999-05-06 by Klaus Peichl                        **
; *************                   Dornbergstr. 10                     **
; *************                   91058 Erlangen-Tennenlohe           **
; *************                   (pei@freenet.de)                    **
; **********************************************************************

		.Model tiny
		.Code
		Org 100h

; ----------------------------------------------------------------------
  INT21         MACRO
		LOCAL Ok
; ----------------------------------------------------------------------
		int 21h
		jnc Ok
		mov dx,Offset FileErrStr
		mov ah,9
		int 21h
		mov ErrorLevel,100
                jmp Ende
	    ok: ENDM


; **********************************************************************
; ************* Hauptprogramm                                         **
; **********************************************************************

Start:          cmp sp,Offset Stapelend
                ja  StapelOk
                mov dx,Offset NoMemStr
                mov ah,9
                int 21h
                jmp Ende

StapelOk:       push ds
		pop es
		cld

; ************* Dateiname vorbereiten **********************************

		mov si,81h
CmdLineLoop:    lodsb
                cmp al,13
                jbe ParamEnd
		cmp al,' '
		je CmdLineLoop

                cmp al,'/'
                jne IsName
                lodsb
		and al,255-32           ; Klein -> Grobuchstabe
                cmp al,'F'              ; <F>ormatierungsstring
                jne n_F
                lodsb
                cmp al,':'
                jne SyntaxError
                mov FormatStrPtr,si
  skipFormat:   lodsb
                cmp al,'A'
                jae skipFormat
                jmp CmdLineLoop
  n_F:          cmp al,'L'
                je  LB
                jmp SyntaxError
  LB:           inc DoLineBreak
                jmp CmdLineLoop

IsName:         mov dx,si               ; Anfangspos. Dateiname
		dec dx

ScanName:       lodsb
		cmp al,' '
		ja ScanName

		dec si
		mov Byte Ptr [si],0

ParamEnd:       cmp si,82h
                ja ParamOk

SyntaxError:    mov ErrorLevel,101
                mov dx,Offset SyntaxStr
                mov ah,9
                int 21h
                jmp Ende
ParamOk:

; ************* Datei einlesen *****************************************
;               Der MP3-Tag belegt die letzten 128 Bytes

		mov ax,3D00h            ; Open File (read)
		INT21

OpenOk:         mov bx,ax               ; bx := Handle

                mov ax,4202h            ; SEEK_END
                mov cx,-1
                mov dx,-128
                INT21

		mov cx,128
		mov dx,Offset Tag
		mov ax,3F00h            ; Daten einlesen
		INT21

                mov ax,3E00h            ; Datei schlieen
		INT21


; ************* Prfen, ob MP3-Tag vorhanden ist ***********************

                mov si,Offset Tag
                cld
                lodsw
                cmp ax,'AT'
                jne no_TAG
                lodsb
                mov al,'G'
                jne no_TAG
                jmp Parse
no_TAG:         jmp Ende


; ************* Format-String auswerten ********************************

Parse:          mov si,FormatStrPtr
ParseLoop:      cld
                lodsb
		and al,255-32           ; Klein -> Grobuchstabe
                cmp al,'A'              ; <A>rtist?
                jb  ParseEnd
                jne n_A
                mov bx,Offset Interpret
                mov dx,Offset Interpret+29
                call Output
                jmp ParseLoop
  n_A:          cmp al,'B'              ; Al<b>um?
                jne n_B
                mov bx,Offset Album
                mov dx,Offset Album+29
                call Output
                jmp ParseLoop
  n_B:          cmp al,'T'              ; <T>itle?
                jne n_T
                mov bx,Offset Titel
                mov dx,Offset Titel+29
                call Output
                jmp ParseLoop
  n_T:          cmp al,'Y'              ; <Y>ear?
                jne n_Y
                mov bx,Offset Jahr
                mov dx,Offset Jahr+3
                call Output
                jmp ParseLoop
  n_Y:          cmp al,'C'              ; <C>omment?
                jne n_C
                mov bx,Offset Kommentar
                mov dx,Offset Kommentar+29
                call Output
                jmp ParseLoop
  n_C:
                jmp SyntaxError

  ParseEnd:     mov ax,Breite
                mov ErrorLevel,al

Ende:           mov al,Errorlevel
		mov ah,4ch
		int 21h

; ----------------------------------------------------------------------
  Output:
; ----------------------------------------------------------------------
; Eingabe: bx = Anfang des Feldes
;          dx = letztes Zeichen im Feld

                push si
                mov si,dx

                ; Tatschliche Lnge des Feldes ermitteln:
                std
WidthLoop:      lodsb
                cmp si,bx
                je OutputEnd            ; Feld ist leer
                cmp al,' '
                jbe WidthLoop
                sub si,bx
                inc si
                inc si                  ; si=Lnge

Trennung:       ; Falls vorher schon ein Feld ausgegeben wurde,
                ; erst den Trenn-String schreiben:
                cmp Breite,0
                je  TrennEnd
                mov dx,Offset CR
                cmp DoLineBreak,1
                jae Trenne
                mov dx,Offset TrennStr
                add Breite,3
  Trenne:       mov ah,9
                int 21h
  TrennEnd:

SetWidth:       ; Gesamt-Breite erhhen bzw. aktualisieren:
                cmp DoLineBreak,1
                jae CheckMax
                add Breite,si
                jmp SetWidthEnd
  CheckMax:     cmp si,Breite
                jbe SetWidthEnd
                mov Breite,si
  SetWidthEnd:

                mov Byte Ptr [bx+si],'$'
                mov dx,bx
                mov ah,9
                int 21h                 ; Feld ausgeben

OutputEnd:      pop si
                ret




; ************************************************************************
; ************* Konstanten und Variablen                                **
; ************************************************************************

SyntaxStr       DB 'SHOWTAG  (c) 1999-05-06...1999-08-15 by Klaus Peichl',10,10,13
                DB 'This tool displays ASCII information that is often appended',10,13
                DB 'to MPEG audio bitstreams (MP3-tag).',10,10,13
		DB 'Syntax:        SHOWTAG [/L] /F:<Format-String> <Filename>',10,13
                DB 'Option:        /L inserts a linebreak instead of a dash between the items',10,13
                DB 'Format-String: May contain A,B,T,C and Y where',10,13
                DB '               A prints out the artist,',10,13
                DB '               B prints out the album,',10,13
                DB '               T prints out the song title,',10,13
                DB '               C prints out the comment and',10,13
                DB '               Y prints out the year of compilation.',10,10,13
                DB 'Example:       SHOWTAG /F:TA CAMBODIA.MP3',10,13
                DB '               => Cambodia - Kim Wilde',10,10,13
                DB 'Errorlevel:    0=no tag, 100=file error, 101=syntax error,',10,13
                DB '               else=width of output',10,13,'$'
NoMemStr        DB '?OUT OF MEMORY  ERROR',10,13,'$'
FileErrStr      DB 'SHOWTAG Fatal: File I/O Error!'
CR              DB 10,13,'$'
TrennStr        DB ' - $'
ErrorLevel      DB 0
DoLineBreak     DB 0
Breite          DW 0
FormatStrPtr    DW ?
Tag             DB  3 Dup(?)
Titel           DB 30 Dup(?)
Interpret       DB 30 Dup(?)
Album           DB 30 Dup(?)
Jahr            DB  4 Dup(?)
Kommentar       DB 30 Dup(?)
Genre           DB  ?
Stapel          DB 500 Dup (?)
StapelEnd       DB ?
		END Start
