; SysError ersetzt den Critical Error Handler von DOS (Abort, Retry, Fail)
; (C) 2000-02-09 by Klaus Peichl, stliche Ringstr. 7, 85113 Bhmfeld
;
; Es gengt nicht, den Handler (Int 24h) einmalig umzuleiten.
; Statt dessen wird Int 21h so manipuliert, dass vor jeder DOS-Funktion
; der Errorhandler erneuert wird.


                .MODEL tiny
                .Code
                ORG 100h

Start:          jmp Installation


; ----------------------------------------------------------------------
  Int_24h:
; ----------------------------------------------------------------------
                push ds
                push ax                 ; Register retten
                push bx
                push dx
                push es
                pushf

                push cs
                pop  ds

                test ah,1
                jz ReadError
                inc WriteErrors
                jmp Return
  ReadError:    inc ReadErrors
Return:         mov al,ReturnWert
                popf
                pop es
                pop dx
                pop bx
                pop ax
                pop ds
                iret

                DB 0EAh                 ; EA=JMP
Ofsalt_24       DW  0227h
Segalt_24       DW 0FF9Fh


; ----------------------------------------------------------------------
  Int_21h:
; ----------------------------------------------------------------------
                push ds
                push ax                 ; Register retten
                push es
                pushf

                push cs
                pop  ds

                xor ax,ax
                mov es,ax
                mov ax,Offset Int_24h
                mov [ES:90h],ax
                mov ax,cs
                mov [ES:92h],ax

                popf
                pop es
                pop ax
                pop ds

                DB 0EAh                 ; EA=JMP
Ofsalt_21       DW  0227h
Segalt_21       DW 0FF9Fh

; **********************************************************************
; ************* Variablen (resident gespeichert)                      **
; **********************************************************************

ReturnWert      DB 255
ReadErrors      DW 0
WriteErrors     DW 0

; ----------------------------------------------------------------------
  Installation:
; ----------------------------------------------------------------------

Parameter_Auswertung:


                mov si,81h
  Hole_Zeichen: lodsb
                cmp al,' '
                je Hole_Zeichen
                cmp al,'/'
                jne no_Slash
                jmp Slash
    no_Slash:   cmp al,13
                ja Parameter
                jmp Residiere

  Parameter:    and al,0DFh             ; Klein->Grobuchstabe
                cmp al,'A'              ; <A>bort
                jne kein_A
                mov ReturnWert,2
                jmp Hole_Zeichen

    kein_A:     cmp al,'F'              ; <F>ail
                jne kein_F
                mov ReturnWert,3
                jmp Hole_Zeichen

    kein_F:     cmp al,'R'              ; <R>etry
                jne kein_R
                mov ReturnWert,1
                jmp Hole_Zeichen

    kein_R:     cmp al,'I'              ; <I>gnore (default)
                jne kein_I
                mov ReturnWert,0
                jmp Hole_Zeichen
    kein_I:

  Syntaxfehler:
                mov dx,Offset Kopf_String
                call Print

                mov dx,Offset Syntax_String
                call Print
                jmp Programm_entfernen

  Slash:        lodsb
                and al,0DFh             ; Klein->Grobuchstabe
                cmp al,'R'              ; <R>emove
                jne n_R
                call CheckInstall
                jnc  IstJaGarNicht
                call DeInstall
                jmp Programm_entfernen
  IstJaGarNicht: mov dx,Offset NichtDaStr
                 call Print
                jmp Programm_entfernen

    n_R:        cmp al,'C'              ; <C>ount
                jne n_C
                call CheckInstall
                jnc noCount
                mov ax,[ES:ReadErrors]
                call PrintHexWord
                mov dx,Offset REcountStr
                call Print
                mov ax,[ES:WriteErrors]
                call PrintHexWord
                mov dx,Offset WEcountStr
                call Print
                jmp Programm_entfernen

     noCount:   mov dx,Offset noCountStr
                call Print
                jmp Programm_entfernen
    n_C:        cmp al,'Q'
                jne n_Q
                mov Quiet,1
                jmp Hole_Zeichen
    n_Q:        jmp Syntaxfehler


; ----------------------------------------------------------------------
  Residiere:
; ----------------------------------------------------------------------
                cmp ReturnWert,255
                je SyntaxFehler

                mov dx,Offset Kopf_String
                call Print

                call CheckInstall
                jnc  nochNichtDa
                call DeInstall          ; falls schon installiert: erst entfernen
  NochNichtDa:  mov dx,Offset Install_Str
                call Print


                mov ax,2521h            ; Interrupt 21h umbiegen
                mov dx,Offset Int_21h
                int 21h


                mov ax,[DS:2ch]
                mov es,ax
                mov ah,49h
                int 21h                 ; Umgebungsbereich freigeben

                                        ; residenten Teil
                mov dx,24               ; ab Installation freigeben
                                        ; dx=Anzahl der im Speicher zu haltenden
                                        ; Paragraphen  16 Byte
                mov ax,3100h            ; resident beenden
                int 21h



; ----------------------------------------------------------------------
  CheckInstall:
; ----------------------------------------------------------------------
; Return:       Carry=0  =>  noch nicht installiert
;               Carry=1  =>  ist schon installiert
;               ES = Segment der installierten Routine

                ; bestehenden Int 24h-Vektor prfen:
                mov ax,3524h            ; aktuellen Int-Vektor lesen
                int 21h
                mov Segalt_24,es
                mov Ofsalt_24,bx

                ; bestehenden Int 21h-Vektor prfen:
                mov ax,3521h            ; aktuellen Int-Vektor lesen
                int 21h
                mov Segalt_21,es
                mov Ofsalt_21,bx

                mov cx,word ptr [ES:bx] ; 1. Wort von aktueller Routine lesen
                cmp cx,word ptr [DS:Int_21h] ; SYSERROR schon installiert?
                jne nicht_installiert

                mov cx,word ptr [ES:bx+10] ; 10. Wort von aktueller Routine lesen
                cmp cx,word ptr [DS:Int_21h+10] ; SYSERROR schon installiert?
                jne nicht_installiert

                stc
                jmp CheckRet
  nicht_installiert:
                clc

  CheckRet:     ret



; ----------------------------------------------------------------------
  Deinstall:
; ----------------------------------------------------------------------
                push ds

                mov dx,ES:Segalt_21     ; ursprngl. Segment holen
                mov Segalt_21,dx        ; als aktuelle Rcksprungadresse
                mov ds,dx
                mov dx,ES:Ofsalt_21     ; ursprngl. Offset holen
                mov CS:Ofsalt_21,dx     ; aktuelle Rcksprungadresse
                mov ax,2521h            ; Int.vektor zurcksetzen
                int 21h

                mov dx,ES:Segalt_24     ; ursprngl. Segment holen
                mov Segalt_24,dx        ; als aktuelle Rcksprungadresse
                mov ds,dx
                mov dx,ES:Ofsalt_24     ; ursprngl. Offset holen
                mov CS:Ofsalt_24,dx     ; aktuelle Rcksprungadresse
                mov ax,2524h            ; Int.vektor zurcksetzen
                int 21h

                mov ah,49h
                int 21h                 ; bisherige Routine rausschmeissen

                pop ds

                mov dx,Offset Uninstall_Str
                call Print
                ret


Programm_entfernen:
                mov ax,004ch
                int 21h


; ----------------------------------------------------------------------
  Print:
; ----------------------------------------------------------------------
; Gibt einen String aus, wenn Quiet nicht 0 ist.
; Eingabe:      dx = Offset des Strings

                cmp Quiet,0
                jnz PrintEnd
                push ax
                mov ah,9
                int 21h
                pop ax
  PrintEnd:     ret


; *************************************************
; *  Hex-Zahlausgabe                              *
; *  Eingabe:         ax = Zahl bei PrintHexWord  *
; *             bzw.  al = Zahl bei PrintHexByte  *
; *************************************************
PrintHexWord:   xchg al,ah
                call PrintHexByte
                xchg al,ah
                call PrintHexByte
                ret

PrintHexByte:   push dx
                mov dl,al
                shr dl,1
                shr dl,1
                shr dl,1
                shr dl,1
		call Ziffer
		mov dl,al
		call Ziffer
		pop dx
		ret

  Ziffer:       and dl,0Fh
                add dl,'0'
		cmp dl,'9'
		jbe Schreibe_Ziffer
                add dl,7
  Schreibe_Ziffer: push ax
		mov ah,2
		int 21h
		pop ax
		ret


; **********************************************************************
; ************* Variablen und Strings (nicht resident)                **
; **********************************************************************


Quiet           DB 0
Kopf_String     DB 'SysError (C) 2000 by Klaus Peichl',10,13,'$'
Syntax_String   DB 10,13,'This tool replaces the DOS critical error handler by a dummy routine that',10,13
                DB 'simply counts the number of errors instead of notifying the user.',10,13
                DB 'The purpose is to avoid the message "(A)bort (R)etry (F)ail (I)gnore".',10,10,13
                DB 'Usage:  SysError [/Q] A  Simulate "Abort" on all errors',10,13
                DB '        SysError [/Q] F  Simulate "Fail" on all errors',10,13
                DB '        SysError [/Q] I  Simulate "Ignore" on all errors',10,13
                DB '        SysError [/Q] R  Simulate "Retry" on all errors',10,10,13
                DB '        SysError /C      Display error counts',10,13
                DB '        SysError /R      Remove SysError from memory',10,13
                DB '        SysError /?      Display this help screen',10,10,13
                DB '        The optional switch /Q (quiet) suppresses any output to the screen',10,13,'$'
Install_Str     DB 'SysError Critical error handler installed.',10,13,'$'
Uninstall_Str   DB 'Resident portion of SysError detected and removed.',10,13,'$'
NichtDaStr      DB 'SysError could not be removed from memory!',10,13,'$'
NoCountStr      DB 'Error counters could not be evaluated!',10,13,'$'
REcountStr      DB '(hex) Read Errors,',10,13,'$'
WEcountStr      DB '(hex) Write Errors.',10,13,'$'
                END Start

